@@ -2,7 +2,6 @@
 #' 
 #' Calculates the SiZer map from a given set of X and Y variables.
 #' 
-#' @usage SiZer(x, y, h=NA, x.grid=NA, degree=NA, derv=1, grid.length=41)
 #' 
 #' @param x data vector for the independent axis
 #' @param y data vector for the dependent axis
@@ -18,6 +17,7 @@
 #' @param derv The order of derivative for which to make the SiZer map.
 #' @param degree The degree of the local weighted polynomial used to smooth the data.
 #'   This must be greater than or equal to \code{derv}.
+#' @param quiet Should diagnostic messages be suppressed? Defaults to TRUE.
 #'   
 #' @details SiZer stands for the Significant Zero crossings of the derivative.  There are two 
 #' dominate approaches in smoothing bivariate data: locally weighted regression or penalized splines.
@@ -57,6 +57,7 @@
 #' # Calculate the SiZer map for the first derivative
 #' SiZer.1 <- SiZer(x, y, h=c(.5,10), degree=1, derv=1, grid.length=21)
 #' plot(SiZer.1)
+#' plot(SiZer.1, ggplot2=TRUE)
 #' 
 #' # Calculate the SiZer map for the second derivative
 #' SiZer.2 <- SiZer(x, y, h=c(.5,10), degree=2, derv=2, grid.length=21);
@@ -68,7 +69,7 @@
 #' # SiZer.3 <- SiZer(x, y, h=c(.5,10), grid.length=100, degree=1, derv=1)
 #' # plot(SiZer.3)  
 #'   
-SiZer <- function(x, y, h=NA, x.grid=NA, degree=NA, derv=1, grid.length=41){
+SiZer <- function(x, y, h=NA, x.grid=NA, degree=NA, derv=1, grid.length=41, quiet=TRUE){
   
   # calculate x.grid and h.grid from what was passed in.
   x.grid <- x.grid.create(x.grid, x, y, grid.length);
@@ -82,7 +83,7 @@ SiZer <- function(x, y, h=NA, x.grid=NA, degree=NA, derv=1, grid.length=41){
 	row <- 1;
 	slopes <- matrix(nrow=length(h.grid), ncol=length(x.grid));
 	for( h in h.grid ){
-		print(h)
+		if(!quiet){ message(paste0('Now calculating h=', h)) }
 		model <- locally.weighted.polynomial(x, y, h=h, x.grid=x.grid, degree=degree);     
   		intervals <- 
   	      calc.CI.LocallyWeightedPolynomial(model, derv=derv);
@@ -107,7 +108,10 @@ SiZer <- function(x, y, h=NA, x.grid=NA, degree=NA, derv=1, grid.length=41){
 #' @param colorlist What colors should be used.  This is a vector that 
 #'        corresponds to 'decreasing', 'possibley zero', 'increasing', 
 #'        and 'insufficient data'.
-#' @param \dots Any other parameters to be passed to the function \code{image}.
+#' @param ggplot2 Should the graphing be done using `ggplot2`? Defaults to 
+#'        FALSE for backwards compatibility. 
+#' @param \dots Any other parameters to be passed to the function \code{image}. 
+#'        Ignored if `ggplot2` is TRUE.
 #' 
 #' @details The white lines in the SiZer map give a graphical representation 
 #'          of the bandwidth.  The horizontal distance between the lines is \eqn{2h}.
@@ -135,6 +139,7 @@ SiZer <- function(x, y, h=NA, x.grid=NA, degree=NA, derv=1, grid.length=41){
 #' # Calculate the SiZer map for the first derivative
 #' SiZer.1 <- SiZer(x, y, h=c(.5,10), degree=1, derv=1, grid.length=21)
 #' plot(SiZer.1)
+#' plot(SiZer.1, ggplot2=TRUE)
 #' 
 #' # Calculate the SiZer map for the second derivative
 #' SiZer.2 <- SiZer(x, y, h=c(.5,10), degree=2, derv=2, grid.length=21);
@@ -147,7 +152,14 @@ SiZer <- function(x, y, h=NA, x.grid=NA, degree=NA, derv=1, grid.length=41){
 #' # plot(SiZer.3)  
 #'   
 plot.SiZer <- function(x, ylab=expression(log[10](h)), 
-			colorlist=c('red', 'purple', 'blue', 'grey'), ...){
+			colorlist=c('red', 'purple', 'blue', 'grey'), 
+			ggplot2=FALSE, ...){
+  
+  if(ggplot2){
+    out <- ggplot_SiZer(x, colorlist=colorlist)
+    return(out)
+  }
+  
 	temp <- factor(x$slopes);
     final.colorlist <- NULL;
 	if( is.element( '-1', levels(temp) ) )
@@ -172,6 +184,85 @@ plot.SiZer <- function(x, ylab=expression(log[10](h)),
 }	
 
 
+#' Plot a SiZer map using `ggplot2`
+#' 
+#' Plot a `SiZer` object that was created using `SiZer()`
+#' 
+#' @param x An object created using `SiZer()`
+#' @param colorlist What colors should be used.  This is a vector that 
+#'        corresponds to 'decreasing', 'possibley zero', 'increasing', 
+#'        and 'insufficient data'.
+#' 
+#' @details The white lines in the SiZer map give a graphical representation 
+#'          of the bandwidth.  The horizontal distance between the lines is \eqn{2h}.
+#'  
+#' @references 
+#'   Chaudhuri, P., and J. S. Marron. 1999. SiZer for exploration of structures
+#'     in curves. Journal of the American Statistical Association 94:807-823. 
+#'     
+#'   Hannig, J., and J. S. Marron. 2006. Advanced distribution theory for SiZer. 
+#'     Journal of the American Statistical Association 101:484-499.
+#'     
+#'   Sonderegger, D.L., Wang, H., Clements, W.H., and Noon, B.R. 2009. Using SiZer to detect
+#'     thresholds in ecological data. Frontiers in Ecology and the Environment 7:190-195.  
+#' 
+#' @author Derek Sonderegger
+#' @seealso \code{\link{plot.SiZer}}, \code{\link{locally.weighted.polynomial}}
+#' @export
+#' @examples 
+#' data('Arkansas')
+#' x <- Arkansas$year
+#' y <- Arkansas$sqrt.mayflies
+#' 
+#' plot(x,y)
+#' 
+#' # Calculate the SiZer map for the first derivative
+#' SiZer.1 <- SiZer(x, y, h=c(.5,10), degree=1, derv=1, grid.length=21)
+#' plot(SiZer.1)
+#' plot(SiZer.1, ggplot2=TRUE)
+#' ggplot_SiZer(SiZer.1)
+#' 
+#' # Calculate the SiZer map for the second derivative
+#' SiZer.2 <- SiZer(x, y, h=c(.5,10), degree=2, derv=2, grid.length=21);
+#' plot(SiZer.2)
+#' plot(SiZer.2, ggplot2=TRUE)
+#' ggplot_SiZer(SiZer.2)
+#' 
+#' 
+#' # By setting the grid.length larger, we get a more detailed SiZer
+#' # map but it takes longer to compute. 
+#' #
+#' # SiZer.3 <- SiZer(x, y, h=c(.5,10), grid.length=100, degree=1, derv=1)
+#' # plot(SiZer.3)  
+#'   
+#' @importFrom rlang .data
+ggplot_SiZer <- function(x, 
+                       colorlist=c('red', 'purple', 'blue', 'grey') ){
+  df <- as.data.frame(x)
+  out <- df |>
+    ggplot2::ggplot( ggplot2::aes(x=x, y=h)) +
+    ggplot2::geom_tile( ggplot2::aes(fill=class)) +
+    ggplot2::scale_y_log10() +
+    ggplot2::scale_fill_manual( values=colorlist ) 
+
+  # draw the bandwidth lines
+  x.midpoint <- diff(range(x$x.grid))/2 + min(x$x.grid)
+  tmp1 <- data.frame(x = x.midpoint + x$h.grid, h = x$h.grid) 
+  tmp2 <- data.frame(x = x.midpoint - x$h.grid, h = x$h.grid) 
+  tmp <- rbind(tmp1, tmp2)
+  
+  # R cmd Check freaks out because it cant see where h is defined
+  # in the tmp data frame, so we need to shut it up somehow.
+  h = NULL  # I can't believe this is a good idea, but whatever.
+  
+  out <- out +
+    ggplot2::geom_line( data=tmp, ggplot2::aes(x=x, y=h), color='white') 
+  
+  return(out)
+}	
+
+
+
 h.grid.create <- function(h.grid, x, y, grid.length){
 	foo <- grid.length;
 	h.max <- diff( range(x) ) * 2;
@@ -193,4 +284,43 @@ h.grid.create <- function(h.grid, x, y, grid.length){
 	return(out);
 }
 
-
+#' Coerce SiZer object to a Data Frame
+#' 
+#' @param x An object produced by `SiZer()`.
+#' @param row.names Required for generic compatibility. Not used.
+#' @param optional  Required for generic compatibility. Not used.
+#' @param ...  Required for generic compatibility. Not used. 
+#' @export
+#' 
+#' @examples 
+#' data('Arkansas')
+#' x <- Arkansas$year
+#' y <- Arkansas$sqrt.mayflies
+#' 
+#' plot(x,y)
+#' 
+#' # Calculate the SiZer map for the first derivative
+#' SiZer.1 <- SiZer(x, y, h=c(.5,10), degree=1, derv=1, grid.length=21)
+#' as.data.frame(SiZer.1)
+#' @importFrom rlang .data
+as.data.frame.SiZer <- function(x, row.names=NULL, optional=FALSE, ...){
+  obj <- x  # too many x running around, but the generic requires x to be the parameter
+  
+  out <- obj$slopes |> 
+    as.data.frame(row.names=row.names, optional=optional, ...) |>
+    dplyr::mutate(h = obj$h.grid) |>
+    tidyr::pivot_longer(cols = dplyr::starts_with('V'), names_to = 'x_bin', values_to='class')
+  
+  # x-values are currently V1 ... V_gridsize. So fix that...
+  tmp <- data.frame(x_bin = paste0('V',1:length(obj$x.grid)),
+                    x     = obj$x.grid)
+  out <- dplyr::left_join(out, tmp, by='x_bin') |>
+    dplyr::select(-.data$x_bin) |>
+    dplyr::relocate(.data$x, .before = 1)
+  
+  # make sure the class value is a factor with all the necessary levels
+  out <- out |>
+    dplyr::mutate(class = factor(class, levels=c(-1:2),
+                                 labels = c('decreasing','flat','increasing','insufficient data')))
+  return(out)
+}
\ No newline at end of file
